!/* Boundary Element Method (_BEM) formulation for 2D Laplace problem(_2D)
!   For constant elements interpolation, nodes local order 
!   on conectivity "elements-nodes"(IE,JE) are:
!                   
!    JE (N): 1  2  
!
!        ---|---|---|---|---|---|---|---|---|---|---
!        |                                         |
!        -                                         -
!        |                                         |
!        -                                         -
!        |                                         |
!        -                                         -
!        |                                         O "local node 2" (BEM element 2)
!        -                                         -
!        |                                         |
!        -                                         -
!        |                                         |
!        -                                         -
!        |                                         |
!        ---|---|---|---|---|---| O |---|---|---|---
!                               "local node 1" (BEM element 1)
!
!   This strategy use both colocation points, node 1 integrating over 
!   "BEM element 2" and node 2 integrating over "BEM element 1".
!
!   
!--------------------------------------------------------------------
!
!  Off-Diagonal assembler BEM-elemet on Matrix (_M)
!  for constant element(_C)
!  Case (101)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      SubRoutine Laplace_BEM_C_M_2D                           &
                ( AE,BE,MaxLRows,XLL,NDim,iDofT,NodELT,       & 
                  Sol0,Sol1,CommonPar,                        &  
                  Param,JParam,DelT,DTm,Time)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      USE Direct_lib
      IMPLICIT REAL*8 (A-H,O-Z)
	  Parameter ( iT=1 )
      DIMENSION AE(MaxLRows,MaxLRows),BE(MaxLRows),                       &
       XLL(1),Sol0(1),Sol1(1),                                            &
       Param(1), JParam(1),CommonPar(1), z0(NDim)

Cond2 = 2.0D0*CommonPar(1); ! difusivity coeficient 

! Assembling Node 1 Matrix Coefficient for "p" an "q" unknows, respectively
  z0(1) = 0.50D0*(XLL(1)+XLL(3))
  z0(2) = 0.50D0*(XLL(2)+XLL(4))
! ecuation asociated to "p" unknown
  AE(1,5) =  DirectF(NDim,z0,XLL(5),XLL(7))
  AE(1,6) = -DirectG(NDim,z0,XLL(5),XLL(7))
! ecuation asociated to "q" unknown
  AE(2,5) = AE(1,5)
  AE(2,6) = AE(1,6)

! Assembling Node 3 Matrix Coefficient for "p" an "q" unknows, respectively
  z0(1) = 0.50D0*(XLL(5)+XLL(7))
  z0(2) = 0.50D0*(XLL(6)+XLL(8))
! ecuation asociated to "p" unknown
  AE(5,1) =  DirectF(NDim,z0,XLL(1),XLL(3))
  AE(5,2) = -DirectG(NDim,z0,XLL(1),XLL(3))
! ecuation asociated to "q" unknown
  AE(6,1) = AE(5,1)
  AE(6,2) = AE(5,2)

  Return
 End


! 
! ---------------------------------------------------------------------
!
!  Diagonal(_D) assembler BEM-elemet for constant element(_C)
!  Case (103)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      SubRoutine Laplace_BEM_C_D_2D                           &
                ( AE,BE,MaxLRows,XLL,NDim,iDofT,NodELT,       & 
                   Sol0,Sol1,CommonPar,                        &  
                  Param,JParam,DelT,DTm,Time)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      USE Direct_lib
      IMPLICIT REAL*8 (A-H,O-Z)
	  Parameter ( iT=1 )
      DIMENSION AE(MaxLRows,MaxLRows),BE(MaxLRows),                       &
       XLL(1),Sol0(1),Sol1(1),                                            &
       Param(1), JParam(1),CommonPar(1), z0(NDim)

Cond2 = CommonPar(1); ! difusivity coeficient 

! Assembling Node 1 Matrix Coefficient for "p" an "q" unknows, respectively
  z0(1) = 0.50D0*(XLL(1)+XLL(3))
  z0(2) = 0.50D0*(XLL(2)+XLL(4))
! ecuation asociated to "p" unknown
  AE(1,1) =  0.50D0
  AE(1,2) = -DirectG(NDim,z0,XLL(1),XLL(3))
! ecuation asociated to "q" unknown
  AE(2,1) = AE(1,1)
  AE(2,2) = AE(1,2)

  Return
 End
